unit edtDBScrollboxOrd;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, 
  StdCtrls, Buttons, ExtCtrls, 
  wwDBEdit, 
  wwDatSrc, 
  wwRiched, 
  Wwdbdlg, 
  wwdblook, 
  Wwdotdot, 
  Wwdbspin, 
  Wwdbcomb, 
  Mask, 
  dbctrls;

type
  TfrmEdtDBScrollBoxOrd = class(TForm)
      Panel1         : TPanel;
      btnOk          : TBitBtn;
      btnMoveUp      : TButton;
      btnMoveDown    : TButton;
      BitBtn1        : TBitBtn;
      lbxDBScrollBox : TListBox;
      procedure lbxDBScrollBoxDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
      procedure FormShow(Sender: TObject);
      procedure FormClose(Sender: TObject; var Action: TCloseAction);
      procedure btnMoveUpClick(Sender: TObject);
      procedure btnMoveDownClick(Sender: TObject);
      procedure lbxDBScrollBoxDragDrop(Sender, Source: TObject; X,
      Y: Integer);
      procedure lbxDBScrollBoxDragOver(Sender, Source: TObject; X,
      Y: Integer; State: TDragState; var Accept: Boolean);
    private
      { Private declarations }
      FBitMap   : TBitMap;
      FTabOrder : integer;
    public
      { Public declarations }
      ControlList : TList;
      FieldList   : TList;
      procedure AddItem (Control : TControl);
  end;

var
  frmEdtDBScrollBoxOrd : TfrmEdtDBScrollBoxOrd;

implementation

{$R *.DFM}

procedure TfrmEdtDBScrollBoxOrd.AddItem (Control : TControl);
var
  displayName : string;
begin
  DisplayName := Control.Name;

  FBitmap := TBitmap.Create;

  if Control is TwwDBEdit           then FBitMap.LoadFromResourceName(HInstance,'TWWDBEDIT');
  if Control is TwwDBComboBox       then FBitMap.LoadFromResourceName(HInstance,'TWWDBCOMBOBOX');
  if Control is TwwDBSpinEdit       then FBitMap.LoadFromResourceName(HInstance,'TWWDBSPINEDIT');
  if Control is TwwDBComboDlg       then FBitMap.LoadFromResourceName(HInstance,'TWWDBCOMBODLG');
  if Control is TwwDBLookUpCombo    then FBitMap.LoadFromResourceName(HInstance,'TWWDBLOOKUPCOMBO');
  if Control is TwwDBLookUpComboDlg then FBitMap.LoadFromResourceName(HInstance,'TWWDBLOOKUPCOMBODLG');
  if Control is TwwDBRichEdit       then FBitMap.LoadFromResourceName(HInstance,'TWWDBRICHEDIT');
  if Control is TDBImage            then FBitMap.LoadFromResourceName(HInstance,'TDBIMAGE');
  if Control is TDBMemo             then FBitMap.LoadFromResourceName(HInstance,'TDBMEMO');
  if Control is TDBCheckBox         then FBitMap.LoadFromResourceName(HInstance,'TDBCHECKBOX');

  lbxDBScrollBox.ItemHeight := FBitmap.Height + 1;
  lbxDBScrollBox.Items.AddObject(DisplayName, TObject(FBitMap));

end;

procedure TfrmEdtDBScrollBoxOrd.lbxDBScrollBoxDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);

var
  Bitmap : TBitmap;      { temporary variable for the items bitmap }
  Offset : Integer;      { text offset width }

begin

  with (Control as TListBox).Canvas do  { draw on control canvas, not on the form }
    begin
      FillRect(Rect);       { clear the rectangle }
      Offset := 2;          { provide default offset }

      Bitmap := TBitmap((Control as TListBox).Items.Objects[Index]);  { get the bitmap }

      if odSelected in State then Bitmap.Canvas.Brush.Color := clHighlight
      else
          Bitmap.Canvas.Brush.Color := lbxDBScrollBox.Color;

      if Bitmap <> nil then
        begin
          BrushCopy(Bounds(Rect.Left + 2, Rect.Top, Bitmap.Width, Bitmap.Height),
          Bitmap, Bounds(0, 0, Bitmap.Width, Bitmap.Height), clRed);  {render bitmap}
          Offset := Bitmap.width + 6;    { add four pixels between bitmap and text}
        end;

      TextOut(Rect.Left + Offset, Rect.Top, (Control as TListBox).Items[Index])  { display the text }
    end;

end;

procedure TfrmEdtDBScrollBoxOrd.FormShow(Sender: TObject);
var
  i : integer;
begin
  FBitmap   := TBitmap.Create;
  FTabOrder := TWinControl(ControlList[1]).TabOrder; // first tab-order
end;

procedure TfrmEdtDBScrollBoxOrd.FormClose(Sender: TObject; var Action: TCloseAction);
var
  i     : integer;
  Order : integer;
begin
  FBitmap.Free;
  Order := FTabOrder;

  if modalResult = mrOk then
    begin
      for i := 0 to ControlList.Count - 1 do
        begin
          if (TControl(ControlList[i]) is TWinControl) then
            begin
              TWinControl(ControlList[i]).TabOrder := Order;
              inc(Order);
            end;
        end;
    end;
end;

procedure TfrmEdtDBScrollBoxOrd.btnMoveUpClick(Sender: TObject);
var
  OldIndex : integer;
  lbxIndex : integer;
  TOrder   : integer;
begin

  if lbxDBScrollBox.ItemIndex > -1 then
    begin

      if lbxDBScrollBox.ItemIndex <> 0 then
        begin

          lbxDBScrollBox.Items.BeginUpdate;

          try
            OldIndex := lbxDBScrollBox.ItemIndex - 1;

            lbxIndex := lbxDBScrollBox.ItemIndex;

            lbxDBScrollBox.items.InsertObject(
            (lbxDBScrollBox.ItemIndex -1),
            lbxDBScrollBox.Items[lbxDBScrollBox.ItemIndex],
            lbxDBScrollBox.Items.Objects[lbxDBScrollBox.ItemIndex]);

            lbxDBScrollBox.Items.Delete(lbxIndex + 1);

            ControlList.Insert((OldIndex * 2) + 4,ControlList[OldIndex * 2]);
            ControlList.Insert((OldIndex * 2) + 5,ControlList[(OldIndex * 2) + 1]);
            ControlList.Delete((OldIndex * 2) + 1);
            ControlList.Delete((OldIndex * 2));

            FieldList.Insert(OldIndex + 2, FieldList[OldIndex]);
            FieldList.Delete(OldIndex);

            lbxDBScrollBox.ItemIndex := OldIndex;
          finally
            lbxDBScrollBox.Invalidate;
            lbxDBScrollBox.Items.EndUpdate;
          end; // end try finally


        end;
    end;



end;

procedure TfrmEdtDBScrollBoxOrd.btnMoveDownClick(Sender: TObject);
var
  OldIndex : integer;

begin
  if lbxDBScrollBox.ItemIndex > -1 then
    begin
      if lbxDBScrollBox.ItemIndex <> lbxDBScrollBox.Items.count - 1 then
        begin
          try
            lbxDBScrollBox.Items.BeginUpdate;

            OldIndex := lbxDBScrollBox.ItemIndex;

            lbxDBScrollBox.items.InsertObject(
            (lbxDBScrollBox.ItemIndex + 2),
            lbxDBScrollBox.Items[lbxDBScrollBox.ItemIndex],
            lbxDBScrollBox.Items.Objects[lbxDBScrollBox.ItemIndex]);

            lbxDBScrollBox.Items.Delete(lbxDBScrollBox.ItemIndex);

            ControlList.Insert((OldIndex * 2) + 4,ControlList[OldIndex * 2]);    // The label
            ControlList.Insert((OldIndex * 2) + 5,ControlList[(OldIndex * 2) + 1]);  // The field

            ControlList.Delete((OldIndex * 2) + 1);
            ControlList.Delete((OldIndex * 2));

            FieldList.Insert(OldIndex + 2, FieldList[OldIndex]);
            FieldList.Delete(OldIndex);
            lbxDBScrollBox.ItemIndex := OldIndex + 1;
          finally
            lbxDBScrollBox.Invalidate;
            lbxDBScrollBox.Items.EndUpdate;
          end; // end try finally


        end;
    end;

end;

procedure TfrmEdtDBScrollBoxOrd.lbxDBScrollBoxDragDrop(Sender,
  Source: TObject; X, Y: Integer);
var
  OldIdx, Idx : Integer;

begin
  OldIdx := lbxDBScrollBox.ItemIndex;
  Idx    := lbxDBScrollBox.ItemAtPos( Point( X, Y ), True );

  if Idx <> -1 then
    begin
      lbxDBScrollBox.Items.BeginUpdate;
      try
        lbxDBScrollBox.Items.Move( OldIdx, Idx );
        lbxDBScrollBox.ItemIndex := Idx;

        if idx < OldIdx then
          begin
            ControlList.Insert( (Idx * 2), ControlList[(OldIdx * 2) + 1]);  // The field
            ControlList.Insert( (Idx * 2) ,ControlList[(OldIdx * 2)+ 1 ]);  // The Label

            ControlList.Delete((OldIdx * 2) + 3);
            ControlList.Delete((OldIdx * 2) + 2);
          end;

        if idx > OldIdx then
          begin
            ControlList.Insert( (Idx * 2) + 2,ControlList[OldIdx * 2]);        // The label
            ControlList.Insert( ((Idx * 2) + 3),ControlList[(OldIdx * 2) + 1]);  // The field

            ControlList.Delete((OldIdx * 2) + 1);
            ControlList.Delete((OldIdx * 2));
          end;

        FieldList.Insert(Idx, FieldList[OldIdx]);
        FieldList.Delete(OldIdx);

      finally
        lbxDBScrollBox.Invalidate;
        lbxDBScrollBox.Items.EndUpdate;
      end;
    end;
end;

procedure TfrmEdtDBScrollBoxOrd.lbxDBScrollBoxDragOver(Sender,
  Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := Sender = Source;
end;

end.
